//+------------------------------------------------------------------+
//|                                                     FFS_Correlation.mq4 |
//+------------------------------------------------------------------+
#property copyright "icm63 (user name at Forex-TSD)"
#property link      "Forex-TSD"

/* 

This is the index selection format
1. EUR/USD, USD/CHF
2. EUR/USD, USD/CHF, GBP/USD
3. EUR/USD, USD/CHF, GBP/USD, USD/JPY
4. EUR/USD, USD/CHF, USD/JPY
5. EUR/USD, GBP/USD, USD/JPY
6. USD/CHF, GBP/USD
7. USD/CHF, GBP/USD, USD/JPY
8. GBP/USD, USD/JPY


*/

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 White

//User Inputs Parameters
extern int IndexType = 3;
extern int cPeriod = 20;

//---- variables
double USDpos[];
double USDneg[];
double USDcorr[];
int NoBars = 512;


int init()
  {
   IndicatorShortName("FFS_Correlation");
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,USDcorr);
   SetIndexLabel(0,"USDcorr");

   return(0);
  }
  //FUNCTIONS
  
  //Correlation Coefficient
  void RunCorrelationFx(double& Data1[], double& Data2[])
  {
  int count = ArraySize(Data1);
  double Temp1[], Temp2[];
  if(ArraySize(Data1) < 2) return(-1);
  if(ArraySize(Data2) < 2) return(-2);
  if(ArraySize(Data1) != ArraySize(Data2)) return(-3);

       for(int i=0; i<count; i++)
        {
            ArrayCopy(Temp1,Data1, i, i, cPeriod);
            ArrayCopy(Temp2,Data2, i, i, cPeriod);
            USDcorr[i] = CorrelationFx(Temp1,Temp2);
         }
  return(0);
  }
  
  //Correlation Function btw two single dimenson arrays
  double CorrelationFx(double& Data1[], double& Data2[])
  {
  double SumX, SumX2, SumY, SumY2, SumXY, SumW, Num, Den, r;
  int count = ArraySize(Data1);
  
  if(ArraySize(Data1) < 2) return(-1);
  if(ArraySize(Data2) < 2) return(-2);
  if(ArraySize(Data1) != ArraySize(Data2)) return(-3);
  
       for(int i=0; i<count; i++)
        {
            SumX += Data1[i];
            SumX2 += Data1[i] * Data1[i];
            SumY += Data2[i];
            SumY2 += Data2[i] * Data2[i];
            SumXY += Data1[i] * Data2[i];
            SumW += 1;
        }
        
        Num = SumXY - (SumX * SumY / SumW);
        Den = MathSqrt((SumX2 - SumX * SumX / SumW) * (SumY2 - SumY * SumY / SumW));
        
        if (Den != 0)
        {
         r = Num/Den;
        }
        else
        {
         r = 0;
        }

  return(r);
  }
  
  
  // Normalise data between 0 and 100
  void Normalise(double& InData[])
  {
   double max, min, range;
   int count = ArraySize(InData);
   if(count<10) return(-9);
   int k, m;
   double Temp[];
   k = ArrayMaximum(InData,NoBars,0);
   max = InData[k];
   m = ArrayMinimum(InData,NoBars,0);
   min = InData[m];
   range = max - min;
   ArrayResize(Temp, count);
   for(int i=0; i<count; i++)
    {
       Temp[i] = NormalizeDouble((((InData[i] - min) / range)*100),2) ;
    }
    ArrayCopy(InData, Temp);
  }
 //------------------------------------
int deinit()
  {   
   return(0);
  }
int start()
  {      
 
     int limit;
     int counted_bars=IndicatorCounted();
     double Data1, Data2, Data3, Data4;
     double GBP0, EUR0, CHF0, JPY0;
     
     
     if(counted_bars<0) return(-1);
     if(counted_bars>0) counted_bars-=10;
     limit=Bars-counted_bars;
     
     GBP0 = iClose("GBPUSD",0,NoBars);
     EUR0 = iClose("EURUSD",0,NoBars);
     CHF0 = iClose("USDCHF",0,NoBars);
     JPY0 = iClose("USDJPY",0,NoBars);
     
     for(int i=0; i<limit; i++)
       {
            Data1=iClose("EURUSD",0,i);
            Data2=iClose("GBPUSD",0,i);
            Data3=iClose("USDCHF",0,i);
            Data4=iClose("USDJPY",0,i);
    
            switch(IndexType)
               {
               case 1:
                     //1. EUR/USD, USD/CHF"
                     USDpos[i]=  (Data1 - EUR0 ) * 10000;
                     USDneg[i]=  (Data3 - CHF0) * 10000;
                     break;
               case 2:
                     //2. EUR/USD, USD/CHF, GBP/USD
                     USDpos[i]=  (Data1 - EUR0 ) * 10000 + (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data3 - CHF0) * 10000;
                     break;
               case 3:
                     //3. EUR/USD, USD/CHF, GBP/USD, USD/JPY
                     USDpos[i]=  (Data1 - EUR0 ) * 10000+ (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data3 - CHF0) * 10000  + (Data4 - JPY0) * 100  ;
                     break;
               case 4:
                     //4. EUR/USD, USD/CHF, USD/JPY
                     USDpos[i]=  (Data1 - EUR0 ) * 10000;
                     USDneg[i]=  (Data3 - CHF0) * 10000  + (Data4 - JPY0) * 100  ;
                     break;
               case 5:
                     //5. EUR/USD, GBP/USD, USD/JPY
                     USDpos[i]=  (Data1 - EUR0 ) * 10000+ (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data4 - JPY0) * 100  ;
                     break;
               case 6:
                     //6. USD/CHF, GBP/USD
                     USDpos[i]=  (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data3 - CHF0) * 10000 ;
                     break;
               case 7:
                     //7. USD/CHF, GBP/USD, USD/JPY
                     USDpos[i]=  (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data3 - CHF0) * 10000  + (Data4 - JPY0) * 100  ;
                     break;  
               case 8:
                     //8. GBP/USD, USD/JPY 
                     USDpos[i]=  (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data4 - JPY0) * 100  ;
                     break;
               default:
                     USDpos[i]=  (Data1 - EUR0 ) * 10000+ (Data2 - GBP0) * 10000 ;
                     USDneg[i]=  (Data3 - CHF0) * 10000  + (Data4 - JPY0) * 100  ;
                     break;
              }
       }
       
         Normalise(USDpos);
         Normalise(USDneg);
         RunCorrelationFx(USDpos, USDneg);

    return(0);
  }
//+------------------------------------------------------------------+